#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <cvirte.h>     /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "rsupv.h"
#include "rsupv_loop_example.h"

static int panelHdl;
static ViSession instrSession;
static ViStatus status;


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)   /* Needed if linking in external compiler; harmless otherwise */
        return -1;  /* out of memory */
    if ((panelHdl = LoadPanel (0, "rsupv_loop_example.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHdl);
    InstallPopup (panelHdl);
    RunUserInterface ();
    return 0;
}

void errorQuit(char* func, ViStatus status) {
    static char errMsg[512];
    static char label[256];
    if (func != 0) {
        sprintf (label, "%s ERROR", func);
    } else {
        strcpy (label, "ERROR");
    }
    rsupv_error_message (instrSession, status, errMsg);
    MessagePopup (label, errMsg);
    exit (0);
}

int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char strBuf[256];
    ViBoolean reset, id;
    int i;
    ViReal64 value;
    ViReal64 volume;
    ViChar   unit[10];

    switch (event)
        {
        case EVENT_COMMIT:
            // initialize and reset to a known state
            GetCtrlVal (panelHdl, PANEL_INSTRDESCR, strBuf);
            GetCtrlVal (panelHdl, PANEL_RESET, &reset);
            GetCtrlVal (panelHdl, PANEL_IDQUERY, &id);
            if ((status = rsupv_init (strBuf, id, reset, &instrSession)) < 0) {
                errorQuit("rsupv_init", status);
            }

           SetCtrlAttribute (panelHdl, PANEL_APPLY, ATTR_DIMMED, 1);
           ClearListCtrl (panelHdl, PANEL_LISTBOX);   
           ProcessDrawEvents();
            // config trigger
            if ((status = rsupv_SetMeasurementMode (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupv_SetMeasurementMode", status);
            }
            
            // config universal analog generator
            if ((status = rsupv_ConfigureGeneratorAnalog (instrSession,
                                                          RSUPV_GEN_CH_21,
                                                          RSUPV_GEN_OUTPUT_UNBAL,
                                                          RSUPV_GEN_IMPEDANCE_10,
                                                          RSUPV_GEN_OUTPUT_COMM_GND,
                                                          RSUPV_GEN_BWIDTH_22,
                                                          RSUPV_GEN_VOLT_RANG_AUTO,
                                                          10.0, RSUPV_UNIT_V)) < 0) {
                errorQuit("rsupv_ConfigureGeneratorAnalog", status);
            }
            // Configure Generator Reference
            if ((status = rsupv_ConfigureGeneratorReference (instrSession, 10.0, RSUPV_UNIT_V, 1.0e3,
                                       RSUPV_UNIT_HZ)) < 0) {
                errorQuit("rsupv_ConfigureGeneratorReference", status);
            }
            
            // Set Generator Function
            if ((status = rsupv_SetGeneratorFunction (instrSession, RSUPV_GEN_FUNC_SINE)) < 0) {
                errorQuit("rsupv_SetGeneratorFunction", status);
            }
            
            // Set Generator Sine Low Distortion
            if ((status = rsupv_SetGeneratorSineLowDistortion (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupv_SetGeneratorSineLowDistortion", status);
            }            
            
            // Set Generator DC Offset
            if ((status = rsupv_SetGeneratorDCOffset (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupv_SetGeneratorDCOffset", status);
            }                
            
            // Set Generator Voltage
            if ((status = rsupv_SetGeneratorVoltage (instrSession, 1, 1.0, RSUPV_UNIT_V)) < 0) {
                errorQuit("rsupv_SetGeneratorVoltage", status);
            }
            
            // Set Generator Frequency
            if ((status = rsupv_SetGeneratorFrequency (instrSession, 1, 1000.0, RSUPV_UNIT_HZ)) < 0) {
                errorQuit("rsupv_SetGeneratorFrequency", status);
            }
            
            // Set Analyzer Channel Input
            if ((status = rsupv_SetAnalyzerChannelInput (instrSession, 1, RSUPV_ANALYZER_CH_INPUT_BAL)) < 0) {
                errorQuit("rsupv_SetAnalyzerChannelInput", status);
            }
           
            // Set Analyzer Ref Impedance
            if ((status = rsupv_SetAnalyzerRefImpedance (instrSession, 600.0)) < 0) {
                errorQuit("rsupv_SetAnalyzerRefImpedance", status);
            }           

            // Set Analyzer Channel Input Range
            if ((status = rsupv_SetAnalyzerChannelInputRange (instrSession, 1, RSUPV_ANALYZER_CH_INPUT_RANGE_AUTO)) < 0) {
                errorQuit("rsupv_SetAnalyzerChannelInputRange", status);
            }
            
            // Set Analyzer Channel Input Common
            if ((status = rsupv_SetAnalyzerChannelInputCommon (instrSession, 1, RSUPV_ANALYZER_CH_INPUT_COMMON_FLOAT)) < 0) {
                errorQuit("rsupv_SetAnalyzerChannelInputCommon", status);
            }
          
            //Generator Ch1: Internal connection of analyzer channel 1 to generator output 1. The analyzer input impedance is 200 kOhm.
            if ((status = rsupv_SetAnalyzerChannelInput (instrSession, 1, RSUPV_ANALYZER_CH_INPUT_BAL)) < 0) {
                errorQuit("rsupv_SetAnalyzerChannelInput", status);
            }
          
		
            // Set Analyzer Level Meas Time
            if ((status = rsupv_SetAnalyzerLevelMeasTime (instrSession, RSUPV_ANALYZER_MEAS_TIME_AFAS)) < 0) {
                errorQuit("rsupv_SetAnalyzerLevelMeasTime", status);
            }
            
            // Configure Analyzer Notch Filter
            if ((status = rsupv_ConfigureAnalyzerNotchFilter (instrSession, RSUPV_ANALYZER_NOTCH_GAIN_OFF,
                                                 RSUPV_ANALYZER_NOTCH_FREQ_FIX, 10.0)) < 0) {
                errorQuit("rsupv_ConfigureAnalyzerNotchFilter", status);
            }            
            
            // Set Analyzer Filter
            if ((status = rsupv_SetAnalyzerFilter (instrSession, 1, RSUPV_ANALYZER_PREFILTER_OFF)) < 0) {
                errorQuit("rsupv_SetAnalyzerFilter", status);
            }
            
            // Set Analyzer Function
            if ((status = rsupv_SetAnalyzerFunction (instrSession, RSUPV_ANALYZER_FUNC_RMS)) < 0) {
                errorQuit("rsupv_SetAnalyzerFunctinstrSessionn", status);
            }
           
           // Set Unit    
            if ((status = rsupv_SetAnalyzerUnit (instrSession, 1, RSUPV_ANALYZER_MEASUREMENT_FUNC, RSUPV_UNIT_DBV))< 0) {
                errorQuit("rsupv_SetAnalyzerUnit", status);
            }
            
            // Set Aux Speaker
            if ((status = rsupv_SetAuxSpeaker (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupv_SetAuxSpeaker", status);
            }

            // Set Aux Monitoring Channel
            if ((status = rsupv_SetAuxMonitoringChannel (instrSession, RSUPV_AUX_CHANNEL_CH1)) < 0) {
                errorQuit("rsupv_SetAuxMonitoringChannel", status);
            }
          
            // Set Generator Volume
            GetCtrlVal (panelHdl, PANEL_VOLUME, &volume);
		   if ((status = rsupv_SetGeneratorVoltage (instrSession, 1, volume, RSUPV_UNIT_V)) < 0) {
          
           		errorQuit("rsupv_SetGeneratorVoltage ", status);
            }
        
		
        
            // For Loop controlling frequency sweep
            for (i = 0; i < 101; i++) {
                if ((status = rsupv_SetGeneratorModulationFrequency(instrSession,pow (10.0, i/50.0+2.0), RSUPV_UNIT_HZ)) < 0) {
                    errorQuit("rsupv_SetGeneratorFrequency", status);
                }
                if ((status = rsupv_StartMeasurementWaitOPC (instrSession, 15000)  < 0)) {
                    errorQuit("rsupv_StartMeasurementWaitOPC ", status);
                }
                if ((rsupv_ReadMeasurementResult (instrSession, 1, RSUPV_MEASUREMENT_FUNCTION, &value, unit)) < 0) {
                    errorQuit("rsupv_ReadMeasurementResult", status);
                }
                // display measured data
                sprintf(strBuf, "%7.1f Hz     %5.2f %s",pow (10.0, i/50.0+2.0),  value, unit);
				InsertListItem (panelHdl, PANEL_LISTBOX, -1, strBuf, i);
				
            }
            
                if ((status = rsupv_SetAuxSpeaker (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupv_SetAuxSpeaker", status);
            }
            
             SetCtrlAttribute (panelHdl, PANEL_APPLY, ATTR_DIMMED, 0);      
            break;
        }
    return 0;
}

int CVICALLBACK quitCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}
